/*
 *  Copyright (c) xlightweb.org, 2008. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.xsocket.DataConverter;
import org.xsocket.connection.IConnection.FlushMode;




/**
 * 
 * bodyless message writer
 *  
 * @author grro@xlightweb.org
 */
final class BodylessMessageWriter extends AbstractBodyWriter {

	private static final Logger LOG = Logger.getLogger(BodylessMessageWriter.class.getName());
	
	private boolean isHeaderWritten = false;

	private final IHttpHeader header;

	
	/**
	 * constructor 
	 * 
	 * @param httpConnection   the http connection
	 * @param header           the header
	 * @throws IOException if an ioe exception occcurs
	 */
	BodylessMessageWriter(AbstractHttpConnection httpConnection, IHttpHeader header) throws IOException {
		super(httpConnection);
		this.header = header;
	}
	
	
	/**
	 * flushes the body data 
	 * 
	 *  @param bodyData   body data
	 *  @param flushMode  the flush mode 
	 */
	public void flush(ByteBuffer[] bodyData, FlushMode flushMode) throws IOException {

	}
	
		
	/**
	 * closes the message writer
	 * 
	 * @throws IOException if an exception occurs
	 */
	public void close() throws IOException {
		
		AbstractHttpConnection con = getConnection();
		
		if (con != null) {
			
			try {
				if (!isHeaderWritten) {
					isHeaderWritten = true;
					
					con.setFlushmode(FlushMode.ASYNC);
					
					con.write(header.toString() + "\r\n");			
					con.flush();
				}
				
			} catch (IOException ioe) {
				if (LOG.isLoggable(Level.FINE)) {
					LOG.fine("error occured by flushing bodyless message writer. Destroying connection. reason " + DataConverter.toString(ioe));
				}
				
				destroy();
			}
		}
		
		super.close();
	}
}